import { Client } from './client';
/**
 * Web3 provider that interacts with EVM contracts deployed on Loom DAppChains.
 */
export declare class LoomProvider {
    private _client;
    private _connection?;
    private _subscriptionID;
    private _topicsList;
    protected notificationCallbacks: Array<Function>;
    protected responseCallbacks: any;
    /**
     * @param client: The client which calls Ethereum EVM
     */
    constructor(client: Client);
    on(type: string, callback: any): void;
    addDefaultEvents(): void;
    removeListener(): void;
    removeAllListeners(type: any): void;
    reset(): void;
    /**
     * Should be used to make async request
     * This method is used internally by web3, so we adapt it to be used with loom contract
     * when we are wrapping the evm on a DAppChain
     * @param payload JSON payload generated by web3 which will be translated to loom transaction/call
     * @param callback Triggered on end with (err, result)
     */
    send(payload: any, callback: Function): Promise<void>;
    private _callAsync(payload);
    private _callStaticAsync(payload);
    private _getReceipt(txHash);
    protected _subscribeWS(readUrl?: string): void;
    protected _onWebSocketMessage(msgEvent: MessageEvent): void;
    protected _timeout(): void;
    private _okResponse(result?);
}
