"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var loom_pb_1 = require("./proto/loom_pb");
var address_1 = require("./address");
var crypto_utils_1 = require("./crypto-utils");
var errors = {
    ErrorResponse: function (result) {
        var message = !!result && !!result.error && !!result.error.message ? result.error.message : JSON.stringify(result);
        return new Error('Returned error: ' + message);
    },
    InvalidNumberOfParams: function (got, expected, method) {
        return new Error('Invalid number of parameters for "' + method + '". Got ' + got + ' expected ' + expected + '!');
    },
    Invalid_Connection: function (host) {
        return new Error('_CONNECTION ERROR: Couldn\'t connect to node ' + host + '.');
    },
    InvalidProvider: function () {
        return new Error('Provider not set or invalid');
    },
    InvalidResponse: function (result) {
        var message = !!result && !!result.error && !!result.error.message ? result.error.message : 'Invalid JSON RPC response: ' + JSON.stringify(result);
        return new Error(message);
    },
    _ConnectionTimeout: function (ms) {
        return new Error('_CONNECTION TIMEOUT: timeout of ' + ms + ' ms achived');
    }
};
/**
 * Web3 provider that interacts with EVM contracts deployed on Loom DAppChains.
 */
var LoomProvider = /** @class */ (function () {
    /**
     * @param client: The client which calls Ethereum EVM
     */
    function LoomProvider(client) {
        this.notificationCallbacks = new Array();
        this._client = client;
        this._subscriptionID = crypto_utils_1.getGUID();
        this._topicsList = [];
        this._subscribeWS(client.readUrl);
    }
    LoomProvider.prototype.on = function (type, callback) {
        if (!this._connection)
            return;
        switch (type) {
            case 'data':
                this.notificationCallbacks.push(callback);
                break;
            case 'connect':
                this._connection.onopen = callback;
                break;
            case 'end':
                this._connection.onclose = callback;
                break;
            case 'error':
                this._connection.onerror = callback;
                break;
        }
    };
    LoomProvider.prototype.addDefaultEvents = function () {
        var _this = this;
        if (!this._connection)
            return;
        this._connection.onerror = function () { return _this._timeout(); };
        this._connection.onclose = function () {
            _this._timeout();
            // reset all requests and callbacks
            _this.reset();
        };
    };
    LoomProvider.prototype.removeListener = function () {
        this.reset();
    };
    LoomProvider.prototype.removeAllListeners = function (type) {
        this.reset();
    };
    LoomProvider.prototype.reset = function () {
        this._topicsList = [];
        this.notificationCallbacks = [];
    };
    /**
     * Should be used to make async request
     * This method is used internally by web3, so we adapt it to be used with loom contract
     * when we are wrapping the evm on a DAppChain
     * @param payload JSON payload generated by web3 which will be translated to loom transaction/call
     * @param callback Triggered on end with (err, result)
     */
    LoomProvider.prototype.send = function (payload, callback) {
        return __awaiter(this, void 0, void 0, function () {
            var okMethods, result, err_1, result, err_2, result, err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        okMethods = ['eth_estimateGas', 'eth_gasPrice'];
                        if (!(okMethods.indexOf(payload.method) !== -1)) return [3 /*break*/, 1];
                        callback(null, this._okResponse());
                        return [3 /*break*/, 17];
                    case 1:
                        if (!(payload.method === 'eth_sendTransaction')) return [3 /*break*/, 6];
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, this._callAsync(payload.params[0])];
                    case 3:
                        result = _a.sent();
                        callback(null, this._okResponse(crypto_utils_1.bytesToHexAddr(result)));
                        return [3 /*break*/, 5];
                    case 4:
                        err_1 = _a.sent();
                        callback(err_1, null);
                        return [3 /*break*/, 5];
                    case 5: return [3 /*break*/, 17];
                    case 6:
                        if (!(payload.method === 'eth_call')) return [3 /*break*/, 11];
                        _a.label = 7;
                    case 7:
                        _a.trys.push([7, 9, , 10]);
                        return [4 /*yield*/, this._callStaticAsync(payload.params[0])];
                    case 8:
                        result = _a.sent();
                        callback(null, this._okResponse(crypto_utils_1.bytesToHexAddr(result)));
                        return [3 /*break*/, 10];
                    case 9:
                        err_2 = _a.sent();
                        callback(err_2, null);
                        return [3 /*break*/, 10];
                    case 10: return [3 /*break*/, 17];
                    case 11:
                        if (!(payload.method === 'eth_getTransactionReceipt')) return [3 /*break*/, 16];
                        _a.label = 12;
                    case 12:
                        _a.trys.push([12, 14, , 15]);
                        return [4 /*yield*/, this._getReceipt(payload.params[0])];
                    case 13:
                        result = _a.sent();
                        callback(null, this._okResponse(result));
                        return [3 /*break*/, 15];
                    case 14:
                        err_3 = _a.sent();
                        callback(err_3, null);
                        return [3 /*break*/, 15];
                    case 15: return [3 /*break*/, 17];
                    case 16:
                        if (payload.method === 'eth_subscribe') {
                            if (payload.params[0] === 'logs') {
                                this._topicsList = this._topicsList.concat(payload.params[1].topics);
                                callback(null, this._okResponse(payload.params[1].topics[0]));
                            }
                            else {
                                callback(null, this._okResponse());
                            }
                        }
                        // Warn the user about we don't support other methods
                        else {
                            callback(Error("Method \"" + payload.method + "\" not supported on this provider"), null);
                        }
                        _a.label = 17;
                    case 17: return [2 /*return*/];
                }
            });
        });
    };
    LoomProvider.prototype._callAsync = function (payload) {
        var caller = new address_1.Address(this._client.chainId, address_1.LocalAddress.fromHexString(payload.from));
        var address = new address_1.Address(this._client.chainId, address_1.LocalAddress.fromHexString(payload.to));
        var data = Buffer.from(payload.data.substring(2), 'hex');
        var callTx = new loom_pb_1.CallTx();
        callTx.setVmType(loom_pb_1.VMType.EVM);
        callTx.setInput(crypto_utils_1.bufferToProtobufBytes(data));
        var msgTx = new loom_pb_1.MessageTx();
        msgTx.setFrom(caller.MarshalPB());
        msgTx.setTo(address.MarshalPB());
        msgTx.setData(callTx.serializeBinary());
        var tx = new loom_pb_1.Transaction();
        tx.setId(2);
        tx.setData(msgTx.serializeBinary());
        return this._client.commitTxAsync(tx);
    };
    LoomProvider.prototype._callStaticAsync = function (payload) {
        var address = new address_1.Address(this._client.chainId, address_1.LocalAddress.fromHexString(payload.to));
        var data = Buffer.from(payload.data.substring(2), 'hex');
        return this._client.queryAsync(address, data, loom_pb_1.VMType.EVM);
    };
    LoomProvider.prototype._getReceipt = function (txHash) {
        return __awaiter(this, void 0, void 0, function () {
            var data, receipt, transactionHash, transactionIndex, blockHash, blockNumber, contractAddress, logs;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        data = Buffer.from(txHash.substring(2), 'hex');
                        return [4 /*yield*/, this._client.getTxReceiptAsync(crypto_utils_1.bufferToProtobufBytes(data))];
                    case 1:
                        receipt = _a.sent();
                        if (!receipt) {
                            throw Error('Receipt cannot be empty');
                        }
                        transactionHash = '0x0000000000000000000000000000000000000000000000000000000000000000';
                        transactionIndex = crypto_utils_1.numberToHex(receipt.getTransactionIndex());
                        blockHash = crypto_utils_1.bytesToHexAddr(receipt.getBlockHash_asU8());
                        blockNumber = crypto_utils_1.numberToHex(receipt.getBlockNumber());
                        contractAddress = crypto_utils_1.bytesToHexAddr(receipt.getContractAddress_asU8());
                        logs = receipt.getLogsList().map(function (logEvent, index) {
                            var logIndex = crypto_utils_1.numberToHex(index);
                            return {
                                logIndex: logIndex,
                                address: contractAddress,
                                blockHash: blockHash,
                                blockNumber: blockNumber,
                                transactionHash: transactionHash,
                                transactionIndex: transactionIndex,
                                type: 'mined',
                                data: crypto_utils_1.bytesToHexAddr(logEvent.getData_asU8()).toLowerCase(),
                                topics: logEvent.getTopicsList_asU8().map(function (topic) { return crypto_utils_1.bytesToHexAddr(topic).toLowerCase(); })
                            };
                        });
                        return [2 /*return*/, {
                                transactionHash: transactionHash,
                                transactionIndex: transactionIndex,
                                blockHash: blockHash,
                                blockNumber: blockNumber,
                                contractAddress: contractAddress,
                                gasUsed: crypto_utils_1.numberToHex(receipt.getGasUsed()),
                                cumulativeGasUsed: crypto_utils_1.numberToHex(receipt.getCumulativeGasUsed()),
                                logs: logs,
                                status: crypto_utils_1.numberToHex(receipt.getStatus()),
                            }];
                }
            });
        });
    };
    LoomProvider.prototype._subscribeWS = function (readUrl) {
        var _this = this;
        if (readUrl) {
            this._connection = new WebSocket(readUrl);
            this._connection.onopen = function () {
                if (!_this._connection)
                    return;
                _this._connection.send(JSON.stringify({
                    method: 'subevents',
                    jsonrpc: '2.0',
                    params: [],
                    id: _this._subscriptionID
                }));
                _this._connection.onmessage = function (msg) { return _this._onWebSocketMessage(msg); };
            };
        }
    };
    LoomProvider.prototype._onWebSocketMessage = function (msgEvent) {
        var _this = this;
        var data = JSON.parse(msgEvent.data);
        var encodedData = data.result.encodedData;
        if (encodedData) {
            var event_1 = loom_pb_1.Event.deserializeBinary(encodedData);
            this.notificationCallbacks.forEach(function (callback) {
                var topics = event_1.getTopicsList_asU8().map(function (topic) { return crypto_utils_1.bytesToHexAddr(topic).toLowerCase(); });
                var topicIdxFound = _this._topicsList.indexOf(topics[0]);
                if (topicIdxFound !== -1) {
                    var topicFound = _this._topicsList[topicIdxFound];
                    var JSONRPCResult = {
                        jsonrpc: "2.0",
                        method: "eth_subscription",
                        params: {
                            // TODO: This ID Should came from loomchain events
                            subscription: topicFound,
                            result: {
                                // TODO: Values bellow should be fix in the future
                                logIndex: '0x00',
                                transactionIndex: '0x00',
                                transactionHash: '0x0000000000000000000000000000000000000000000000000000000000000000',
                                blockHash: '0x0000000000000000000000000000000000000000000000000000000000000000',
                                blockNumber: '0x0',
                                address: '0x0000000000000000000000000000000000000000',
                                type: 'mined',
                                data: crypto_utils_1.bytesToHexAddr(event_1.getData_asU8()).toLowerCase(),
                                topics: topics
                            }
                        }
                    };
                    callback(JSONRPCResult);
                }
            });
        }
    };
    LoomProvider.prototype._timeout = function () {
        for (var key in this.responseCallbacks) {
            if (this.responseCallbacks.hasOwnProperty(key)) {
                this.responseCallbacks[key](errors.Invalid_Connection('on WS'));
                delete this.responseCallbacks[key];
            }
        }
    };
    // Basic response to web3js
    LoomProvider.prototype._okResponse = function (result) {
        if (result === void 0) { result = 0; }
        return { id: 0, jsonrpc: '2.0', result: result };
    };
    return LoomProvider;
}());
exports.LoomProvider = LoomProvider;
//# sourceMappingURL=loom-provider.js.map